# Author:   cBox
# Email:    william@jwawa.com
#
# 2002.11

import re
from StructuredText import STletters, StructuredText, ClassicDocumentClass, DocumentClass

if len(STletters.letters) < 180:
	for i in range(127, 256):
		STletters.letters += chr(i)

StructuredText.letters = STletters.letters
ClassicDocumentClass.letters = STletters.letters
DocumentClass.letters = STletters.letters

letters = STletters.letters
digits = STletters.digits
literal_punc = STletters.literal_punc
phrase_delimiters = STletters.phrase_delimiters
strongem_punc = STletters.strongem_punc
under_punc = STletters.under_punc
dbl_quoted_punc = STletters.dbl_quoted_punc

def doc_numbered(
	self, paragraph,
	expr = re.compile(r'(\s*[%s]\.)|(\s*[0-9]+\.)|(\s*[0-9]+\s+)' % letters).match):

	# This is the old expression. It had a nasty habit
	# of grabbing paragraphs that began with a single
	# letter word even if there was no following period.

	#expr = re.compile('\s*'
	#				   '(([a-zA-Z]|[0-9]+|[ivxlcdmIVXLCDM]+)\.)*'
	#				   '([a-zA-Z]|[0-9]+|[ivxlcdmIVXLCDM]+)\.?'
	#				   '\s+').match):

	top=paragraph.getColorizableTexts()[0]
	m=expr(top)
	if not m: return None
	subs=paragraph.getSubparagraphs()
	if top[-2:]=='::':
		subs=[DocumentClass.StructuredTextExample(subs)]
		top=top[:-1]
	return DocumentClass.StructuredTextNumbered(top[m.span()[1]:], subs,
									indent=paragraph.indent,
									number=top[:m.span()[1]])

DocumentClass.DocumentClass.doc_numbered = doc_numbered

def doc_literal(
	self, s,
	expr = re.compile(r"(\W+|^)'([%s%s%s\s]+)'([%s]+|$)" % (letters, digits, literal_punc, phrase_delimiters)).search,):

	# old expr... failed to cross newlines.
	#		expr=re.compile(
	#		  r"(?:\s|^)'"		   # open
	#		  r"([^ \t\n\r\f\v']|[^ \t\n\r\f\v'][^\n\r']*[^ \t\n\r\f\v'])" # contents
	#		  r"'(?:\s|[,.;:!?]|$)"  # close
	#		  ).search):

	r=expr(s) #or expr2(s)
	if r:
		start, end = r.span(2)
		return (DocumentClass.StructuredTextLiteral(s[start:end]), start-1, end+1)
	else:
		return None

DocumentClass.DocumentClass.doc_literal = doc_literal

def doc_emphasize(
	self, s,
	expr = re.compile(r'\*([%s%s%s\s]+?)\*' % (letters, digits, strongem_punc)).search
	#expr = re.compile(r'\s*\*([ \n\r%s0-9.:/;,\'\"\?\-\_\/\=\-\>\<\(\)]+)\*(?!\*|-)' % letters).search	   # old expr, inconsistent punctuation
	):

	r=expr(s)
	if r:
		start, end = r.span(1)
		return (DocumentClass.StructuredTextEmphasis(s[start:end]), start-1, end+1)
	else:
		return None

DocumentClass.DocumentClass.doc_emphasize = doc_emphasize

def doc_inner_link(self,
				   s,
				   expr1 = re.compile(r"\.\.\s*").search,
				   expr2 = re.compile(r"\[[%s%s]+\]" % (letters, digits) ).search):

	# make sure we dont grab a named link
	if expr2(s) and expr1(s):
		start1,end1 = expr1(s).span()
		start2,end2 = expr2(s).span()
		if end1 == start2:
			# uh-oh, looks like a named link
			return None
		else:
			# the .. is somewhere else, ignore it
			return (DocumentClass.StructuredTextInnerLink(s[start2+1:end2-1]),start2,end2)
		return None
	elif expr2(s) and not expr1(s):
		start,end = expr2(s).span()
		return (DocumentClass.StructuredTextInnerLink(s[start+1:end-1]),start,end)
	return None

DocumentClass.DocumentClass.doc_inner_link = doc_inner_link

def doc_named_link(self,
				   s,
				   expr=re.compile(r"(\.\.\s)(\[[%s0-9]+\])" % letters).search):

	result = expr(s)
	if result:
		start,end   = result.span(2)
		str = s[start+1:end-1]
		st,en	   = result.span()
		return (DocumentClass.StructuredTextNamedLink(str),st,en)
	return None

DocumentClass.DocumentClass.doc_named_link = doc_named_link

def doc_underline(self,
				  s,
				  expr=re.compile(r'_([%s%s%s\s]+)_([\s%s]|$)' % (letters, digits, under_punc,phrase_delimiters)).search):

	result = expr(s)
	if result:
		if result.group(1)[:1] == '_':
			return None # no double unders
		start,end = result.span(1)
		st,e = result.span()
		return (DocumentClass.StructuredTextUnderline(s[start:end]),st,e-len(result.group(2)))
	else:
		return None

DocumentClass.DocumentClass.doc_underline = doc_underline

def doc_strong(self,
			   s,
			   expr = re.compile(r'\*\*([%s%s%s\s]+?)\*\*' % (letters, digits, strongem_punc)).search
			   #expr = re.compile(r'\s*\*\*([ \n\r%s0-9.:/;,\'\"\?\-\_\/\=\-\>\<\(\)]+)\*\*(?!\*|-)' % letters).search, # old expr, inconsistent punc, failed to cross newlines.
	):
	r=expr(s)
	if r:
		start, end = r.span(1)
		return (DocumentClass.StructuredTextStrong(s[start:end]), start-2, end+2)
	else:
		return None

DocumentClass.DocumentClass.doc_strong = doc_strong

_DQUOTEDTEXT = r'("[ %s0-9\n\r%s]+")' % (letters,dbl_quoted_punc) ## double quoted text
_ABSOLUTE_URL=r'((http|https|ftp|mailto|file|about)[:/]+?[%s0-9_\@\.\,\?\!\/\:\;\-\#\~\=\&\%%\+]+)' % letters
_ABS_AND_RELATIVE_URL=r'([%s0-9_\@\.\,\?\!\/\:\;\-\#\~\=\&\%%\+]+)' % letters
_SPACES = r'(\s*)'

def doc_href1(self, s,
			  expr=re.compile(_DQUOTEDTEXT + "(:)" + _ABS_AND_RELATIVE_URL + _SPACES).search
			   ):
	return self.doc_href(s, expr)

DocumentClass.DocumentClass.doc_href1 = doc_href1

def doc_href2(self, s,
			  expr=re.compile(_DQUOTEDTEXT + r'(\,\s+)' + _ABSOLUTE_URL + _SPACES).search
			   ):
	return self.doc_href(s, expr)

DocumentClass.DocumentClass.doc_href2 = doc_href2

def doc_sgml(self,s,expr=re.compile(r"\<[%s0-9\.\=\'\"\:\/\-\#\+\s\*]+\>" % letters).search):
	"""
	SGML text is ignored and outputed as-is
	"""
	r = expr(s)
	if r:
		start,end = r.span()
		text = s[start:end]
		return (DocumentClass.StructuredTextSGML(text),start,end)

DocumentClass.DocumentClass.doc_sgml = doc_sgml

def doc_xref(self, s,
	expr = re.compile('\[([%s0-9\-.:/;,\n\r\~]+)\]' % letters).search
	):
	r = expr(s)
	if r:
		start, end = r.span(1)
		return (DocumentClass.StructuredTextXref(s[start:end]), start-1, end+1)
	else:
		return None

DocumentClass.DocumentClass.doc_xref = doc_xref
